/* $Workfile: OsaLinuxC.c $   $Revision: 6 $    $Modtime: 11/01/04 3:40p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2003, All Rights Reserved

*******************************************************************************/

#include "OsaLinuxC.h"
#include "OsaLinuxRtl.h"
#include "OsaLinuxCpp.h"  

/*---------------------------------------------------------------------------*\  
 from OsaLinuxCppRtl.h OsaLinuxCppRtl.cpp
\*---------------------------------------------------------------------------*/

void OsaSpinLockImpl_Ctor(void **pSLock)
{
  if (sizeof(spinlock_t) > 0) {
    *pSLock = kmalloc(sizeof(spinlock_t), GFP_KERNEL);
    if (*pSLock) {
      spin_lock_init((spinlock_t *) *pSLock);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaSpinLockImpl_Ctor!\n"));
    }
  } else {
    spin_lock_init((spinlock_t *) *pSLock);
  }
}

void OsaSpinLockImpl_Dtor(void **pSLock)
{
  if (sizeof(spinlock_t) > 0) {
    if (*pSLock) {
      kfree(*pSLock);
      *pSLock = NULL;
    }
  }
}

void OsaSpinLockImpl_Lock(void *pSLock, __u32 *flags)
{
  __u32 lflags;

  spin_lock_irqsave((spinlock_t *) pSLock, lflags);
  *flags = lflags;
}

void OsaSpinLockImpl_Unlock(void *pSLock, __u32 savedFlags)
{
  spin_unlock_irqrestore((spinlock_t *) pSLock, savedFlags);
}

void OsaEventImpl_Ctor(void **pWQHead)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
  if (sizeof(wait_queue) > 0) {
    *pWQHead = kmalloc(sizeof(wait_queue), GFP_KERNEL);
    if (*pWQHead) {
      init_waitqueue_head((wait_queue *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaEventImpl_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue *) *pWQHead);
  }
#else
  if (sizeof(wait_queue_head_t) > 0) {
    *pWQHead = kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);
    if (*pWQHead) {
      init_waitqueue_head((wait_queue_head_t *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaEventImpl_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue_head_t *) *pWQHead);
  }
#endif
}

void OsaEventImpl_Dtor(void **pWQHead)
{
  if (*pWQHead) {

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
    if (sizeof(wait_queue) > 0) {
      kfree(*pWQHead);
    }
#else
    if (sizeof(wait_queue_head_t) > 0) {
      kfree(*pWQHead);
    }
#endif

  }
  *pWQHead = NULL;
}


void OsaEventImpl_wake_up(void **pWQHead)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
  wake_up((wait_queue **) pWQHead);
#else
  wake_up((wait_queue_head_t *) *pWQHead);
#endif
}


void OsaSemaphoreImpl_Ctor(void **pSemaphore)
{
  *pSemaphore = kmalloc(sizeof(struct semaphore), GFP_KERNEL);
  if (*pSemaphore) {
    sema_init((struct semaphore *) *pSemaphore, 0);     // initial count 0
  } else {
    OSA_EPRINTF(("OsaLinux: Out of memory OsaSemaphoreImpl_Ctor!\n"));
  }
}

void OsaSemaphoreImpl_Dtor(void **pSemaphore)
{
  if (*pSemaphore) {
    kfree(*pSemaphore);
    *pSemaphore = NULL;
  }
}

void OsaSemaphoreImpl_Release(void **pSemaphore)
{
  up((struct semaphore *) *pSemaphore);
}

void OsaSemaphoreImpl_WaitForSemaphore(void **pSemaphore)
{
  down((struct semaphore *) *pSemaphore);
}


/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/
  
void OsaLinuxUserEventDevice_Ctor(void **pWQHead)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
  if (sizeof(wait_queue) > 0) {
    *pWQHead = kmalloc(sizeof(wait_queue), GFP_KERNEL);
    if (*pWQHead) {
      init_waitqueue_head((wait_queue *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaLinuxUserEventDevice_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue *) *pWQHead);
  }
#else
  if (sizeof(wait_queue_head_t) > 0) {
    *pWQHead = kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);
    if (*pWQHead) {
      init_waitqueue_head((wait_queue_head_t *) *pWQHead);
    } else {
      OSA_EPRINTF(("OsaLinux: Out of memory OsaLinuxUserEventDevice_Ctor!\n"));
    }
  } else {
    init_waitqueue_head((wait_queue_head_t *) *pWQHead);
  }
#endif
}

void OsaLinuxUserEventDevice_Dtor(void **pWQHead)
{
  if (*pWQHead) {

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
    if (sizeof(wait_queue) > 0) {
      kfree(*pWQHead);
    }
#else
    if (sizeof(wait_queue_head_t) > 0) {
      kfree(*pWQHead);
    }
#endif

  }
  *pWQHead = NULL;
}

void OsaLinuxUserEventDevice_wake_up(void **pWQHead)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)    
  wake_up((wait_queue **) pWQHead);
#else
  wake_up((wait_queue_head_t *) *pWQHead);
#endif
}

/*---------------------------------------------------------------------------*\  
 from OsaLinux.h and OsaLinux.cpp
\*---------------------------------------------------------------------------*/

int OsaLinux_pci_present()
{
  return pci_present();
}


void OsaLinux_pci_read_config_word(struct pci_dev_otx *dev, int where, u16 *val)
{
  pci_read_config_word((struct pci_dev *)dev, where, val);
}


void OsaLinux_pci_read_config_dword(struct pci_dev_otx *dev, int where, u32 *val)
{
  pci_read_config_dword((struct pci_dev *)dev, where, val);
}


void OsaLinux_pci_write_config_dword(struct pci_dev_otx *dev, int where, u32 val)
{
  pci_write_config_dword((struct pci_dev *)dev, where, val);
}


struct pci_dev_otx * OsaLinux_pci_dev_g_fun1(void)
{
  return (struct pci_dev_otx *)pci_dev_g(pci_devices.next);
}


struct pci_dev_otx * OsaLinux_pci_dev_g_fun2(void)
{
  return (struct pci_dev_otx *)pci_dev_g(&pci_devices);
}


struct pci_dev_otx * OsaLinux_pci_dev_g_fun3(struct pci_dev_otx *pPciDevice)
{
  return (struct pci_dev_otx *) pPciDevice->global_list.next;  // casting 
}


/*---------------------------------------------------------------------------*\
  from OsaLinux.h and OsaLinux.cpp

  for RedHat 9.0; in RedHat 8.0, it compiles without modifying followings
\*---------------------------------------------------------------------------*/

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20) 
  #include <linux/module.h>  
#endif

void OsaLinux_MOD_INC_USE_COUNT(void)
{
   MOD_INC_USE_COUNT;
}


void OsaLinux_MOD_DEC_USE_COUNT(void)
{
   MOD_DEC_USE_COUNT;
}


const void * Osalinux_inter_module_get(const char * sModule)
{
  return inter_module_get(sModule);
}


void OsaLinux_inter_module_register(const char * sName, const void * pFunName)
{
  inter_module_register(sName, THIS_MODULE, pFunName);
}


void OsaLinux_inter_module_unregister(const char * sName)
{
  inter_module_unregister(sName);
}
